import lxml.etree as ET


def make_pb(n, faks_url, faks_id):
    """ returns a tei:pb
    """
    pb_el = ET.Element("{http://www.tei-c.org/ns/1.0}pb")
    pb_el.attrib['n'] = f"{n}"
    pb_el.attrib['facs'] = f"{faks_url}"
    pb_el.attrib[
        "{http://www.w3.org/XML/1998/namespace}id"
    ] = f"faks__{faks_id}"

    return pb_el



import glob
import os
import collections
import lxml.etree as ET
from slugify import slugify

from freud_api_crawler.xml import XMLReader
from freud_api_crawler.freud_api_crawler import TEI_DUMMY


def make_div_list(glob_pattern):
    """defaultdict of tei:divs fetched from XML/TEI matching the past in glob pattern
    :param glob_pattern: a glob pattern to fetch XML/TEI

    :return: default dict with grouped and ordered div-items
    :rtype: defaultdict
    """

    files = glob.glob(glob_pattern)
    divs = []
    for x in files:
        doc = XMLReader(x)
        title = doc.tree.xpath(
            ".//tei:title[@type='manifestation']/text()", namespaces=doc.nsmap
        )[0]
        pub_id = f'{doc.tree.xpath(".//tei:rs[1]/@ref", namespaces=doc.nsmap)[0]}'
        pub_title = doc.tree.xpath(".//tei:rs[1]/text()", namespaces=doc.nsmap)[0]
        for div in doc.tree.xpath(".//tei:div", namespaces=doc.nsmap):
            first_page = div.xpath(".//tei:pb[@n]/@n", namespaces=doc.nsmap)[0]
            item = {
                "title": title,
                "pub_id": pub_id,
                "pub_title": pub_title,
                "first_page": first_page,
                "div": div
            }
            divs.append(item)

    sorted_list = sorted(divs, key=lambda i: (i['pub_id'], i['title']))
    d = collections.defaultdict(list)
    for x in sorted_list:
        d[x['pub_id']].append(x)
    return d


def create_united_files(glob_pattern):
    output_dir = glob_pattern.replace("*.xml", 'merged')
    try:
        os.makedirs(output_dir)
    except Exception as e:
        pass
    d = make_div_list(glob_pattern)
    xml_obj = XMLReader(TEI_DUMMY)
    for key, value in d.items():
        slug_name = slugify(f"{value[0]['pub_title']}")
        pub_id = f"{key[7:]}"
        doc = ET.parse(TEI_DUMMY)
        save_path = os.path.join(output_dir, f"{slug_name}.xml")
        root_el = doc.xpath('//tei:TEI', namespaces=xml_obj.nsmap)[0]
        root_el.attrib["{http://www.w3.org/XML/1998/namespace}base"] = f"https://whatever.com"
        root_el.attrib[
            "{http://www.w3.org/XML/1998/namespace}id"
        ] = f"manifestation__{pub_id}"
        title = doc.xpath('//tei:title[@type="manifestation"]', namespaces=xml_obj.nsmap)[0]
        t_p = doc.xpath('//tei:title[@type="publication"]', namespaces=xml_obj.nsmap)[0]
        t_p.text = f"{value[0]['pub_title']}"
        body = doc.xpath('//tei:body', namespaces=xml_obj.nsmap)[0]
        for item in value:
            body.append(item['div'])
        with open(f'{save_path}', 'wb') as f:
            f.write(ET.tostring(doc, encoding="UTF-8"))
    return (output_dir, d)



"""Console script for freud_api_crawler."""
import os
import glob
import sys
import click

from . import freud_api_crawler as frd
from . post_process import create_united_files


@click.command()
@click.argument('user', envvar='FRD_USER')
@click.argument('pw', envvar='FRD_PW')
@click.option('-m', default='a10e8c78-adad-4ca2-bfcb-b51bedcd7b58', show_default=True)
def cli(user, pw, m):
    """Console script for freud_api_crawler."""

    auth_items = frd.get_auth_items(user, pw)
    frd_manifestation = frd.FrdManifestation(
        auth_items=auth_items,
        manifestation_id=m
    )
    xml = frd_manifestation.make_xml(save=True)
    click.echo(
        click.style(
            f"processed Manifestation\n###\n {frd_manifestation.md__title}\
            {frd_manifestation.manifestation_id}\n###", fg='green'
        )
    )


@click.command()  # pragma: no cover
@click.argument('user', envvar='FRD_USER')  # pragma: no cover
@click.argument('pw', envvar='FRD_PW')  # pragma: no cover
@click.option('-w', default='9d035a03-28d7-4013-adaf-63337d78ece4', show_default=True)  # pragma: no cover
@click.option('-s', default='/home/csae8092/freud_data_cli', show_default=True)  # pragma: no cover
def download_work(user, pw, w, s):  # pragma: no cover
    """Console script to download all manifestations of a singel work."""
    auth_items = frd.get_auth_items(user, pw)
    werk_obj = frd.FrdWerk(
        auth_items=auth_items, werk_id=w
    )
    rel_manifestations = werk_obj.manifestations
    for x in rel_manifestations:
        try:
            frd_man = frd.FrdManifestation(
                out_dir=s,
                manifestation_id=x['man_id'],
                auth_items=auth_items
            )
            frd_man.make_xml(save=True, limit=False)
        except Exception as e:
            click.echo(
                click.style(
                    f"processing Manifestation {x} did not not work due to Error {e}",
                    fg='red'
                )
            )
    click.echo(
        click.style(
            f"finished download\n{werk_obj.manifestations_count} Manifestations for {werk_obj.md__title} into {s}",
            fg='green'
        )
    )


@click.command()  # pragma: no cover
@click.option('-s', default='/home/csae8092/freud_data_cli/werke/drei-abhandlungen-zur-sexualtheorie', show_default=True)  # pragma: no cover
def merge_files(s):  # pragma: no cover
    """Console script merge splitted manifestaions into single files"""
    glob_pattern = f"{s}/*.xml"
    files = glob.glob(glob_pattern)
    merged = create_united_files(glob_pattern)
    click.echo(
        click.style(
            f"finished: merged {len(files)} Documents from {s}\n\
            into {len(merged[1].keys())} Documents to {merged[0]}",
            fg='green'
        )
    )
